#!/usr/bin/env python

# hard_bevel Rel 2
# Created by Tin Tran
# Comments directed to http://gimpchat.com or http://gimpscripts.com
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: Edited to run in GIMP 2.10.
#import math
#import string
#import Image
from gimpfu import *
#from array import array

def python_hard_bevel(image, layer, azimuth, shadowlength):
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	
	#resize to image size, this also discards text information
	pdb.gimp_layer_resize_to_image_size(layer)
	
	#fill with middle gray
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,layer)
	pdb.gimp_context_set_foreground((128,128,128))
	pdb.gimp_edit_fill(layer,FOREGROUND_FILL)
	
	#saves to channel to be used as bump map later
	pdb.gimp_selection_feather(image,15)
	channel = pdb.gimp_selection_save(image)
	
	#bump map
	pdb.gimp_selection_none(image)
	pdb.plug_in_bump_map(image,layer,channel,azimuth,27,65,0,0,0,0.5,True,False,0)
	
	position = pdb.gimp_image_get_item_position(image,layer)
	shadow1 = pdb.gimp_layer_new_from_drawable(layer,image)
	pdb.gimp_image_insert_layer(image,shadow1,None,position+1)
	pdb.gimp_layer_set_lock_alpha(shadow1,True)
	pdb.gimp_context_set_foreground((0,0,0))
	pdb.gimp_edit_fill(shadow1,FOREGROUND_FILL)
	pdb.gimp_layer_set_lock_alpha(shadow1,False)
	pdb.gimp_item_set_name(shadow1,"Shadow 1")
	#it's mapped differently, the angle passed in to mblur.
	if azimuth <= 180:
		azi = -azimuth
	else:
		azi = 360-azimuth
	pdb.plug_in_mblur(image,shadow1,0,shadowlength,azi%360,0,0)
	
	position = pdb.gimp_image_get_item_position(image,shadow1)
	shadow2 = pdb.gimp_layer_new_from_drawable(shadow1,image)
	pdb.gimp_image_insert_layer(image,shadow2,None,position+1)
	pdb.gimp_item_set_name(shadow2,"Shadow 2")
	
	#color balance
	pdb.gimp_color_balance(layer,1,True,10,0,-10)
	pdb.gimp_color_balance(layer,2,True,5,-15,-30)
	
	pdb.gimp_selection_none(image)
	
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_hard_bevel",                           
	"Creates a hard bevel effect",
	"Creates a hard bevel effect",
	"Tin Tran",
	"Tin Tran",
	"April 2017",
	"<Image>/Python-Fu/Hard Bevel...",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_SPINNER, "azimuth", "Azimuth (angle light coming from):", 45, (0, 360, 1)),
	(PF_SPINNER, "shadowlength", "Shadow Length:", 25, (0, 200, 1)),
	#(PF_COLOR, "oldcolor",  "Replace Color:",  (43,198,255)),
	#(PF_COLOR, "newcolor",  "With New Color:",  (255,198,0)),
	#(PF_SPINNER, "x1", "x1:", 0, (0, 100, 0.5)),
	#(PF_SPINNER, "y1", "y1:", 100, (0, 100, 0.5)),
	#(PF_SPINNER, "x2", "x2:", 100, (0, 100, 0.5)),
	#(PF_SPINNER, "y2", "y2:", 0, (0, 100, 0.5)),
	#(PF_COLOR, "black",  "Black point color",  (0,0,0) PF_FILE, "infilename", "Temp Filepath", "/Default/Path"),
	#(PF_COLOR, "white",  "White point color",  (255,255,255) ),
	#(PF_COLOR, "gray",  "Gray point color",  (128,128,128) )
	#(PF_FILE, "infilename", "Temp Filepath", "/Default/Path")
	#(PF_DIRNAME, "source_directory", "Source Directory", "") for some reason, on my computer when i(Tin) use PF_DIRNAME the pythonw.exe would crash
	],
	[],
	python_hard_bevel)

main()
